(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
	typeof define === 'function' && define.amd ? define(factory) :
	(global.redeuceur = factory());
}(this, (function () { 'use strict';

var compose = function compose(a, b) {
  return function (x) {
    return a(b(x));
  };
};

var identity = function identity(x) {
  return x;
};

var includes = function includes(x, arr) {
  return arr.indexOf(x) > -1;
};

var isArray = function isArray(x) {
  return typeOf(x) === "array";
};

var isFunction = function isFunction(x) {
  return typeOf(x) === "function";
};

var isString = function isString(x) {
  return typeOf(x) === "string";
};

var isOneOfTypes = function isOneOfTypes(types, x) {
  return includes(typeOf(x), types);
};

var typeOf = function typeOf(x) {
  return {}.toString.call(x).match(/\s([a-z|A-Z]+)/)[1].toLowerCase();
};

var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();













var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

function validateHandler(handler) {
  if (!isArray(handler) || handler.length < 2) {
    throw new Error('Invalid handler. Every handler must have a condition and a nextState.');
  }

  return handler;
}

function validateCondition(handler) {
  var validTypes = ['array', 'function', 'string'];
  var isValidCondition = isOneOfTypes(validTypes, handler[0]);

  if (!isValidCondition) {
    throw new TypeError('Invalid handler condition. ' + 'A condition must be a string, an array of strings, or a function.');
  }

  return handler;
}

var defaultHandler = [function () {
  return true;
}, identity];

function redeuceur(initialState) {
  for (var _len = arguments.length, handlers = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    handlers[_key - 1] = arguments[_key];
  }

  handlers.forEach(compose(validateCondition, validateHandler));

  handlers = [].concat(toConsumableArray(handlers), [defaultHandler]);

  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
    var action = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var handler = handlers.find(function (_ref) {
      var _ref2 = slicedToArray(_ref, 1),
          condition = _ref2[0];

      var isHandler = false;

      if (isString(condition)) {
        isHandler = condition === action.type;
      }

      if (isArray(condition)) {
        isHandler = includes(action.type, condition);
      }

      if (isFunction(condition)) {
        isHandler = condition(state, action);
      }

      return isHandler;
    });

    var nextState = isFunction(handler[1]) ? handler[1](state, action) : handler[1];

    return nextState;
  };
}

return redeuceur;

})));
