'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _util = require('./util.js');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function validateHandler(handler) {
  if (!(0, _util.isArray)(handler) || handler.length < 2) {
    throw new Error('Invalid handler. Every handler must have a condition and a nextState.');
  }

  return handler;
}

function validateCondition(handler) {
  var validTypes = ['array', 'function', 'string'];
  var isValidCondition = (0, _util.isOneOfTypes)(validTypes, handler[0]);

  if (!isValidCondition) {
    throw new TypeError('Invalid handler condition. ' + 'A condition must be a string, an array of strings, or a function.');
  }

  return handler;
}

var defaultHandler = [function () {
  return true;
}, _util.identity];

function redeuceur(initialState) {
  for (var _len = arguments.length, handlers = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    handlers[_key - 1] = arguments[_key];
  }

  handlers.forEach((0, _util.compose)(validateCondition, validateHandler));

  handlers = [].concat(_toConsumableArray(handlers), [defaultHandler]);

  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
    var action = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var handler = handlers.find(function (_ref) {
      var _ref2 = _slicedToArray(_ref, 1),
          condition = _ref2[0];

      var isHandler = false;

      if ((0, _util.isString)(condition)) {
        isHandler = condition === action.type;
      }

      if ((0, _util.isArray)(condition)) {
        isHandler = (0, _util.includes)(action.type, condition);
      }

      if ((0, _util.isFunction)(condition)) {
        isHandler = condition(state, action);
      }

      return isHandler;
    });

    var nextState = (0, _util.isFunction)(handler[1]) ? handler[1](state, action) : handler[1];

    return nextState;
  };
}

exports.default = redeuceur;